% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.seqcox.R
\name{print.seqcox}
\alias{print.seqcox}
\title{Print method for seqcox objects}
\usage{
\method{print}{seqcox}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{seqcox} returned by \code{\link[=seqcox]{seqcox()}}.}

\item{digits}{Number of decimal places for printed coefficient summaries.}

\item{...}{Ignored.}
}
\value{
The input object \code{x}, invisibly. Called for its side effect of printing a summary to the console.
}
\description{
Print method for seqcox objects
}
