% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.coxivtsrif.R
\name{print.coxivtsrif}
\alias{print.coxivtsrif}
\title{Print the output from a \code{coxivtsrif} model fit.}
\usage{
\method{print}{coxivtsrif}(x, digits = 3L, ...)
}
\arguments{
\item{x}{an object of class \code{"coxivtsrif"}, usually created by
\code{coxiv_tsrif()}.}

\item{digits}{the number of decimal places to display for coefficient
summaries and hazard ratios.}

\item{...}{additional arguments (currently unused).}
}
\value{
The input object \code{x}, invisibly. Called for its side effect of printing a summary to the console.
}
\description{
Provide succinct and organized output for a \code{coxivtsrif} model, such as
\code{coxiv_tsrif()}.
}
\examples{
data("VitD", package = "surviv")
fit_tsrif <- surviv::coxiv_tsrif(
  surv = "time",
  cens = "death",
  covs = c("age"),
  trt = "vitd",
  iv = "filaggrin",
  tchange = NULL,
  data = VitD,
  fdist = "gaussian",
  bootvar = FALSE,
  B = 20,
  tvareff = FALSE
)
print(fit_tsrif)

}
