% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VitDdata.R
\docType{data}
\name{VitD}
\alias{VitD}
\title{VitD: cohort data on vitamin D and mortality}
\format{
A data frame with variables:
\itemize{
\item \code{age} : Age at baseline
\item \code{filaggrin} : Indicator of filaggrin mutation
\item \code{vitd} : Serum 25-OH-D (nmol/L) at baseline
\item \code{time} : Follow-up time
\item \code{death} : Death indicator during follow-up
}
}
\source{
Mirrored from \pkg{ivtools} (LGPL (>= 3)).
}
\usage{
data(VitD)
}
\description{
This dataset is mirrored from the \pkg{ivtools} package (version 2.3.0).
The upstream documentation notes the data originate from a real cohort study
and were modified for public availability.
}
\references{
Sjolander, Arvid and Martinussen, Torben. "Instrumental Variable Estimation with the R Package ivtools" Epidemiologic Methods, vol. 8, no. 1, 2019, pp. 20180024.
}
\keyword{datasets}
