% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.coxivomom.R
\name{print.coxivomom}
\alias{print.coxivomom}
\title{Print the output from a \code{coxivomom} model fit.}
\usage{
\method{print}{coxivomom}(x, digits = 3L, ...)
}
\arguments{
\item{x}{an object of class \code{"coxivomom"}, usually created by
\code{coxiv_omom()}.}

\item{digits}{the number of decimal places to display for coefficient
summaries and hazard ratios.}

\item{...}{additional arguments (currently unused).}
}
\value{
The input object \code{x}, invisibly. Called for its side effect of printing a summary to the console.
}
\description{
Provide succinct and organized output for a \code{coxivomom} model, such as
\code{coxiv_omom()}.
}
\examples{
data("VitD", package = "surviv")
fit_omom <- surviv::coxiv_omom(
  formula = survival::Surv(time, death) ~ vitd + age,
  trt = "vitd",
  iv = "filaggrin",
  data = VitD
)
print(fit_omom)

}
