% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vasculardata.R
\docType{data}
\name{vascular}
\alias{vascular}
\title{vascular: longitudinal vascular surgery follow-up data}
\format{
A data frame with variables:
\itemize{
\item \code{id} : Unique patient identifier
\item \code{surv.time} : Observed overall follow-up time to death or censoring
\item \code{death} : Terminal event indicator
\item \code{time.start} : Interval start time
\item \code{time.stop} : Interval stop time
\item \code{event} : Counting-process event indicator for the interval
\item \code{reint} : Time-varying reintervention status at the current visit
\item \code{endoleak} : Time-varying endoleak status at the current visit
\item \code{iv} : Baseline instrumental variable for reintervention propensity
\item \code{smoke} : Baseline smoking status indicator
\item \code{age} : Baseline age
\item \code{diameter} : Baseline aneurysm diameter
\item \code{gender} : Baseline gender indicator
\item \code{hyper} : Baseline hypertension indicator
\item \code{obese} : Baseline obesity indicator
\item \code{reintlag1} : Lagged reintervention status from the previous interval
\item \code{visit} : Visit number
}
}
\source{
Synthetic data generated to mimic the distributional features of a
vascular surgery longitudinal follow-up study.
}
\usage{
data(vascular)
}
\description{
A synthetic longitudinal dataset designed to mimic repeated follow-up in a
vascular surgery study. The data are stored in long start-stop format for
survival analyses with a time-varying exposure and a time-varying binary
prognostic factor. In particular, \code{reint} indicates reintervention surgery
status over time and \code{endoleak} indicates time-varying endoleak status over
time. Both are coded so that once the status becomes 1 it remains 1 at later
visits.
}
\keyword{datasets}
